'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = connect;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

require('whatwg-fetch');

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utilsIsPlainObject = require('../utils/isPlainObject');

var _utilsIsPlainObject2 = _interopRequireDefault(_utilsIsPlainObject);

var _PromiseState = require('../PromiseState');

var _PromiseState2 = _interopRequireDefault(_PromiseState);

var _hoistNonReactStatics = require('hoist-non-react-statics');

var _hoistNonReactStatics2 = _interopRequireDefault(_hoistNonReactStatics);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var defaultMapPropsToRequestsToProps = function defaultMapPropsToRequestsToProps() {
  return {};
};

function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}

// Helps track hot reloading.
var nextVersion = 0;

function connect(mapPropsToRequestsToProps) {
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var finalMapPropsToRequestsToProps = mapPropsToRequestsToProps || defaultMapPropsToRequestsToProps;
  var _options$withRef = options.withRef;
  var withRef = _options$withRef === undefined ? false : _options$withRef;

  // Helps track hot reloading.
  var version = nextVersion++;

  function computeRequests(props) {
    var urlsOrRequests = finalMapPropsToRequestsToProps(props) || {};

    _invariant2['default'](_utilsIsPlainObject2['default'](urlsOrRequests), '`mapPropsToRequestsToProps` must return an object. Instead received %s.', urlsOrRequests);

    var requests = {};
    Object.keys(urlsOrRequests).forEach(function (prop) {
      var urlOrRequest = urlsOrRequests[prop];
      if (typeof urlOrRequest === 'string') {
        requests[prop] = new window.Request(urlOrRequest, { credentials: 'same-origin' });
      } else if (urlOrRequest instanceof window.Request) {
        requests[prop] = urlOrRequest;
      } else {
        _invariant2['default'](false, 'Value of `%s` must be either a string or Request. Instead received %s', prop, urlOrRequest);
      }
    });

    return requests;
  }

  function handleResponse(response) {
    if (response.status >= 200 && response.status < 300) {
      return response.json();
    } else {
      return response.json().then(function (errorJson) {
        var id = errorJson.id;
        var error = errorJson.error;
        var message = errorJson.message;

        if (error) {
          throw new Error(error, id);
        } else if (message) {
          throw new Error(message, id);
        } else {
          throw new Error(errorJson);
        }
      });
    }
  }

  return function wrapWithConnect(WrappedComponent) {
    var RefetchConnect = (function (_Component) {
      _inherits(RefetchConnect, _Component);

      function RefetchConnect(props, context) {
        _classCallCheck(this, RefetchConnect);

        _Component.call(this, props, context);
        this.version = version;
        this.state = { requests: {}, data: {} };
      }

      RefetchConnect.prototype.componentWillMount = function componentWillMount() {
        this.refreshData();
      };

      RefetchConnect.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        this.refreshData(nextProps);
      };

      RefetchConnect.prototype.render = function render() {
        var ref = withRef ? 'wrappedInstance' : null;
        return _react2['default'].createElement(WrappedComponent, _extends({}, this.state.data, this.props, { ref: ref }));
      };

      RefetchConnect.prototype.getWrappedInstance = function getWrappedInstance() {
        _invariant2['default'](withRef, 'To access the wrapped instance, you need to specify ' + '{ withRef: true } as the fourth argument of the connect() call.');

        return this.refs.wrappedInstance;
      };

      RefetchConnect.prototype.refreshData = function refreshData() {
        var _this = this;

        var props = arguments.length <= 0 || arguments[0] === undefined ? this.props : arguments[0];

        var nextRequests = computeRequests(props);
        Object.keys(nextRequests).forEach(function (prop) {
          var prev = _this.state.requests[prop];
          var next = nextRequests[prop];
          var comp = ['url', 'method'];
          var same = prev && next && comp.every(function (c) {
            return prev[c] === next[c];
          });

          if (!same) {
            _this.refreshDatum(prop, next);
          }
        });
      };

      RefetchConnect.prototype.refreshDatum = function refreshDatum(prop, request) {
        var _this2 = this;

        this.setPromiseState(prop, request, new _PromiseState2['default']({
          pending: true
        }));

        window.fetch(request).then(handleResponse).then(function (value) {
          _this2.setPromiseState(prop, request, new _PromiseState2['default']({
            fulfilled: true,
            value: value
          }));
        })['catch'](function (error) {
          _this2.setPromiseState(prop, request, new _PromiseState2['default']({
            rejected: true,
            reason: error
          }));
        });
      };

      RefetchConnect.prototype.setPromiseState = function setPromiseState(prop, request, promiseState) {
        this.setState(function (prevState) {
          var _Object$assign, _Object$assign2;

          return {
            requests: Object.assign(prevState.requests, (_Object$assign = {}, _Object$assign[prop] = request, _Object$assign)),
            data: Object.assign(prevState.data, (_Object$assign2 = {}, _Object$assign2[prop] = promiseState, _Object$assign2))
          };
        });
      };

      return RefetchConnect;
    })(_react.Component);

    RefetchConnect.displayName = 'Refetch.connect(' + getDisplayName(WrappedComponent) + ')';
    RefetchConnect.WrappedComponent = WrappedComponent;

    if (process.env.NODE_ENV !== 'production') {
      RefetchConnect.prototype.componentWillUpdate = function componentWillUpdate() {
        if (this.version === version) {
          return;
        }

        // We are hot reloading!
        this.version = version;

        this.refreshData();
      };
    }

    return _hoistNonReactStatics2['default'](RefetchConnect, WrappedComponent);
  };
}

module.exports = exports['default'];